@@ -1,8 +1,8 @@
 /*
+ * Copyright © 2009 Filippo Argiolas <filippo.argiolas@gmail.com>
  * Copyright © 2007,2008 Jaap Haitsma <jaap@haitsma.org>
  * Copyright © 2007-2009 daniel g. siegel <dgsiegel@gnome.org>
  * Copyright © 2008 Ryan Zeigler <zeiglerr@gmail.com>
- * Copyright © 2009 Filippo Argiolas <filippo.argiolas@gmail.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -26,16 +26,33 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 
 #include "cheese-camera-device.h"
 
-G_DEFINE_TYPE (CheeseCameraDevice, cheese_camera_device, G_TYPE_OBJECT)
+static void     cheese_camera_device_initable_iface_init (GInitableIface  *iface);
+static gboolean cheese_camera_device_initable_init       (GInitable       *initable,
+                                                          GCancellable    *cancellable,
+                                                          GError         **error);
+
+G_DEFINE_TYPE_WITH_CODE (CheeseCameraDevice, cheese_camera_device, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
+                                                cheese_camera_device_initable_iface_init))
 
 #define CHEESE_CAMERA_DEVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHEESE_TYPE_CAMERA_DEVICE, \
                                                                           CheeseCameraDevicePrivate))
 
-GST_DEBUG_CATEGORY (cheese_camera_device);
-#define GST_CAT_DEFAULT cheese_camera_device
+#define CHEESE_CAMERA_DEVICE_ERROR cheese_camera_device_error_quark ()
+
+enum CheeseCameraDeviceError
+{
+  CHEESE_CAMERA_DEVICE_ERROR_UNKNOWN,
+  CHEESE_CAMERA_DEVICE_ERROR_NOT_SUPPORTED,
+  CHEESE_CAMERA_DEVICE_ERROR_UNSUPPORTED_CAPS
+};
+
+GST_DEBUG_CATEGORY (cheese_camera_device_cat);
+#define GST_CAT_DEFAULT cheese_camera_device_cat
 
 static gchar *supported_formats[] = {
   "video/x-raw-rgb",
@@ -43,27 +60,36 @@ static gchar *supported_formats[] = {
   NULL
 };
 
+/* FIXME: make this configurable */
+#define CHEESE_MAXIMUM_RATE 30 /* fps */
+
 enum
 {
   PROP_0,
   PROP_NAME,
   PROP_FILE,
   PROP_ID,
-  PROP_SRC
+  PROP_API
 };
 
 typedef struct
 {
   gchar *device;
   gchar *id;
-  gchar *src;
+  const gchar *src;
   gchar *name;
-
+  gint api;
   GstCaps *caps;
-
   GList *formats;
+
+  GError *construct_error;
 } CheeseCameraDevicePrivate;
 
+GQuark
+cheese_camera_device_error_quark (void)
+{
+  return g_quark_from_static_string ("cheese-camera-device-error-quark");
+}
 
 /* CheeseVideoFormat */
 
@@ -111,15 +137,30 @@ cheese_webcam_device_filter_caps (CheeseCameraDevice *device, const GstCaps *cap
 {
 /*  CheeseCameraDevicePrivate *priv =
  *  CHEESE_CAMERA_DEVICE_GET_PRIVATE (device); */
-  gchar   *formats_string = g_strjoinv ("; ", formats);
-  GstCaps *filter         = gst_caps_from_string (formats_string);
-  GstCaps *allowed        = gst_caps_intersect (caps, filter);
+  GstCaps *filter;
+  GstCaps *allowed;
+  gint i;
+
+  filter = gst_caps_new_simple (formats[0],
+                                "framerate", GST_TYPE_FRACTION_RANGE,
+                                0, 1, CHEESE_MAXIMUM_RATE, 1,
+                                NULL);
+
+  for (i = 1; i < g_strv_length (formats); i++)
+  {
+    gst_caps_append (filter,
+                     gst_caps_new_simple (formats[i],
+                                          "framerate", GST_TYPE_FRACTION_RANGE,
+                                          0, 1, CHEESE_MAXIMUM_RATE, 1,
+                                          NULL));
+  }
+
+  allowed = gst_caps_intersect (caps, filter);
 
   GST_DEBUG ("Supported caps %" GST_PTR_FORMAT, caps);
   GST_DEBUG ("Filter caps %" GST_PTR_FORMAT, filter);
   GST_DEBUG ("Filtered caps %" GST_PTR_FORMAT, allowed);
 
-  g_free (formats_string);
   gst_caps_unref (filter);
 
   return allowed;
@@ -168,11 +209,6 @@ cheese_webcam_device_update_format_table (CheeseCameraDevice *device)
   int i;
   int num_structures;
 
-  /* FIXME: limit maximum framerate somehow, we don't want those
-   * stupid 100Hz webcams that slow down the whole pipeline for a
-   * merely perceivable refresh reate gain. I'd say let's throw away
-   * everything over 30/1 */
-
   free_format_list (device);
 
   num_structures = gst_caps_get_size (priv->caps);
@@ -267,7 +303,7 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
 
     /* Check if any error messages were posted on the bus */
     bus = gst_element_get_bus (pipeline);
-    msg = gst_bus_poll (bus, GST_MESSAGE_ERROR, 0);
+    msg = gst_bus_pop_filtered (bus, GST_MESSAGE_ERROR);
     gst_object_unref (bus);
 
     if ((msg == NULL) && (ret == GST_STATE_CHANGE_SUCCESS))
@@ -282,7 +318,15 @@ cheese_camera_device_get_caps (CheeseCameraDevice *device)
       pad        = gst_element_get_pad (src, "src");
       caps       = gst_pad_get_caps (pad);
       priv->caps = cheese_webcam_device_filter_caps (device, caps, supported_formats);
-      cheese_webcam_device_update_format_table (device);
+      if (!gst_caps_is_empty (priv->caps))
+        cheese_webcam_device_update_format_table (device);
+      else {
+        g_set_error_literal (&priv->construct_error,
+                             CHEESE_CAMERA_DEVICE_ERROR,
+                             CHEESE_CAMERA_DEVICE_ERROR_UNSUPPORTED_CAPS,
+                             _("Device capabilities not supported"));
+
+      }
 
       gst_object_unref (pad);
       gst_caps_unref (caps);
@@ -301,7 +345,10 @@ static void
 cheese_camera_device_constructed (GObject *object)
 {
   CheeseCameraDevice *device = CHEESE_CAMERA_DEVICE (object);
+  CheeseCameraDevicePrivate *priv   =
+    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
 
+  priv->src = (priv->api == 2) ? "v4l2src" : "v4lsrc";
 
   cheese_camera_device_get_caps (device);
 
@@ -327,8 +374,8 @@ cheese_camera_device_get_property (GObject *object, guint prop_id, GValue *value
     case PROP_ID:
       g_value_set_string (value, priv->id);
       break;
-    case PROP_SRC:
-      g_value_set_string (value, priv->src);
+    case PROP_API:
+      g_value_set_int (value, priv->api);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -360,10 +407,8 @@ cheese_camera_device_set_property (GObject *object, guint prop_id, const GValue
         g_free (priv->device);
       priv->device = g_value_dup_string (value);
       break;
-    case PROP_SRC:
-      if (priv->src)
-        g_free (priv->src);
-      priv->src = g_value_dup_string (value);
+    case PROP_API:
+      priv->api = g_value_get_int (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -380,7 +425,6 @@ cheese_camera_device_finalize (GObject *object)
 
   g_free (priv->device);
   g_free (priv->id);
-  g_free (priv->src);
   g_free (priv->name);
 
   gst_caps_unref (priv->caps);
@@ -394,6 +438,11 @@ cheese_camera_device_class_init (CheeseCameraDeviceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  if (cheese_camera_device_cat == NULL)
+    GST_DEBUG_CATEGORY_INIT (cheese_camera_device_cat,
+			     "cheese-camera-device",
+			     0, "Cheese Camera Device");
+
   object_class->finalize     = cheese_camera_device_finalize;
   object_class->get_property = cheese_camera_device_get_property;
   object_class->set_property = cheese_camera_device_set_property;
@@ -414,22 +463,25 @@ cheese_camera_device_class_init (CheeseCameraDeviceClass *klass)
                                                         NULL, NULL, NULL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-  g_object_class_install_property (object_class, PROP_SRC,
-                                   g_param_spec_string ("src",
-                                                        NULL, NULL, NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property (object_class, PROP_API,
+                                   g_param_spec_int ("api", NULL, NULL,
+                                                     1, 2, 2,
+                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_type_class_add_private (klass, sizeof (CheeseCameraDevicePrivate));
 }
 
+static void
+cheese_camera_device_initable_iface_init (GInitableIface *iface)
+{
+  iface->init = cheese_camera_device_initable_init;
+}
+
 static void
 cheese_camera_device_init (CheeseCameraDevice *device)
 {
   CheeseCameraDevicePrivate *priv =
     CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
 
-  GST_DEBUG_CATEGORY_INIT (cheese_camera_device,
-                           "cheese-camera-device",
-                           0, "Cheese Camera Device");
   priv->device = NULL;
   priv->id     = NULL;
   priv->src    = NULL;
@@ -437,12 +489,54 @@ cheese_camera_device_init (CheeseCameraDevice *device)
   priv->caps   = gst_caps_new_empty ();
 
   priv->formats = NULL;
+
+  priv->construct_error = NULL;
+}
+
+static gboolean
+cheese_camera_device_initable_init (GInitable *initable,
+                                    GCancellable *cancellable,
+                                    GError **error)
+{
+  CheeseCameraDevice *device = CHEESE_CAMERA_DEVICE (initable);
+  CheeseCameraDevicePrivate *priv =
+    CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
+
+  g_return_val_if_fail (CHEESE_IS_CAMERA_DEVICE (initable), FALSE);
+
+  if (cancellable != NULL)
+  {
+    g_set_error_literal (error,
+                         CHEESE_CAMERA_DEVICE_ERROR,
+                         CHEESE_CAMERA_DEVICE_ERROR_NOT_SUPPORTED,
+                         _("Cancellable initialization not supported"));
+    return FALSE;
+  }
+
+  if (priv->construct_error)
+  {
+    if (error)
+      *error = g_error_copy (priv->construct_error);
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 CheeseCameraDevice *
-cheese_camera_device_new (void)
+cheese_camera_device_new (const gchar *device_id,
+                          const gchar *device_file,
+                          const gchar *product_name,
+                          gint         api_version,
+                          GError **error)
 {
-  return g_object_new (CHEESE_TYPE_CAMERA_DEVICE, NULL);
+  return CHEESE_CAMERA_DEVICE (g_initable_new (CHEESE_TYPE_CAMERA_DEVICE,
+                                               NULL, error,
+                                               "device-id", device_id,
+                                               "device-file", device_file,
+                                               "name", product_name,
+                                               "api", api_version,
+                                               NULL));
 }
 
 /* public methods */
@@ -508,21 +602,22 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
 {
   CheeseCameraDevicePrivate *priv =
     CHEESE_CAMERA_DEVICE_GET_PRIVATE (device);
-  GstCaps *caps;
+  GstCaps *desired_caps;
+  GstCaps *subset_caps;
   gint     i;
 
   GST_INFO ("Getting caps for %dx%d", format->width, format->height);
 
-  caps = gst_caps_new_simple (supported_formats[0],
-                              "width", G_TYPE_INT,
-                              format->width,
-                              "height", G_TYPE_INT,
-                              format->height,
-                              NULL);
+  desired_caps = gst_caps_new_simple (supported_formats[0],
+                                      "width", G_TYPE_INT,
+                                      format->width,
+                                      "height", G_TYPE_INT,
+                                      format->height,
+                                      NULL);
 
   for (i = 1; i < g_strv_length (supported_formats); i++)
   {
-    gst_caps_append (caps,
+    gst_caps_append (desired_caps,
                      gst_caps_new_simple (supported_formats[i],
                                           "width", G_TYPE_INT,
                                           format->width,
@@ -531,13 +626,10 @@ cheese_camera_device_get_caps_for_format (CheeseCameraDevice *device,
                                           NULL));
   }
 
-  if (!gst_caps_can_intersect (caps, priv->caps))
-  {
-    gst_caps_unref (caps);
-    caps = gst_caps_new_empty ();
-  }
+  subset_caps = gst_caps_intersect (desired_caps, priv->caps);
+  gst_caps_unref (desired_caps);
 
-  GST_INFO ("Got %" GST_PTR_FORMAT, caps);
+  GST_INFO ("Got %" GST_PTR_FORMAT, subset_caps);
 
-  return caps;
+  return subset_caps;
 }
